//------------------------------------------------------------------
// Stochastic Momentum Index developed by William Blau
//------------------------------------------------------------------
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1     LimeGreen
#property indicator_color2     PaleVioletRed
#property indicator_color3     PaleVioletRed
#property indicator_color4     Goldenrod
#property indicator_width1     2
#property indicator_width2     2
#property indicator_width3     2
#property indicator_levelcolor MediumOrchid

//
//
//
//
//

extern int    SMIPeriod1      = 13;
extern int    SMIPeriod2      = 25;
extern int    SMIPeriod3      =  2;
extern int    SignalPeriod    =  5;
extern bool   ColorOnSlope    = true;
extern double levelOb         = 60;   
extern double levelOs         = -60;   
extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = false;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = false;
extern bool   alertsEmail     = false;
extern bool   alertsNotify    = false;
extern string soundFile       = "alert2.wav";

//
//
//
//
//

double SMIBuffer[];
double SMIBufferDa[];
double SMIBufferDb[];
double SignalBuffer[];
double colors[];
double trend[];

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(6);
   SetIndexBuffer(0,SMIBuffer);
   SetIndexBuffer(1,SMIBufferDa);
   SetIndexBuffer(2,SMIBufferDb);
   SetIndexBuffer(3,SignalBuffer);
   SetIndexBuffer(4,colors);
   SetIndexBuffer(5,trend);
   SetLevelValue(0,levelOb);
   SetLevelValue(1,levelOs);
   IndicatorShortName("SMI ("+SMIPeriod1+","+SMIPeriod2+","+SMIPeriod3+","+SignalPeriod+")");
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double work[][4];
#define _ema00 0
#define _ema01 1
#define _ema10 2
#define _ema11 3

int start()
{
   int i,r,counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         if (ArrayRange(work,0)!=Bars) ArrayResize(work,Bars);
         if (colors[limit]==-1) CleanPoint(limit,SMIBufferDa,SMIBufferDb);

   //
   //
   //
   //
   //
   
      double alfaS = 2.0 / (1.0 + SignalPeriod);
      double alfa1 = 2.0 / (1.0 + SMIPeriod2);
      double alfa2 = 2.0 / (1.0 + SMIPeriod3);

      for (i=limit, r=Bars-i-1; i>=0; i--,r++)
      {
         double highest = High[iHighest(NULL,0,MODE_HIGH,SMIPeriod1,i)];
         double lowest  = Low [iLowest (NULL,0,MODE_LOW ,SMIPeriod1,i)];
         double sm      = Close[i]-(highest+lowest)*0.5;
         double hl      =           highest-lowest;

            work[r][_ema00] = work[r-1][_ema00]+alfa1*(sm             -work[r-1][_ema00]);
            work[r][_ema01] = work[r-1][_ema01]+alfa2*(work[r][_ema00]-work[r-1][_ema01]);
            work[r][_ema10] = work[r-1][_ema10]+alfa1*(hl             -work[r-1][_ema10]);
            work[r][_ema11] = work[r-1][_ema11]+alfa2*(work[r][_ema10]-work[r-1][_ema11]);
            if (work[r][_ema11] != 0)
                  SMIBuffer[i] = 100.00*(work[r][_ema01]/(0.5*work[r][_ema11]));
            else  SMIBuffer[i] = 0;
            SignalBuffer[i] = SignalBuffer[i+1] + alfaS*(SMIBuffer[i]-SignalBuffer[i+1]);
            
            //
            //
            //
            //
            //
            
            SMIBufferDa[i] = EMPTY_VALUE;
            SMIBufferDb[i] = EMPTY_VALUE;
            colors[i] = colors[i+1];
            trend[i]  = trend[i+1];
               if (ColorOnSlope)
               {
                  if (SMIBuffer[i]>SMIBuffer[i+1]) colors[i]= 1;
                  if (SMIBuffer[i]<SMIBuffer[i+1]) colors[i]=-1;
               }
               else
               {
                  if (SMIBuffer[i]>SignalBuffer[i+1]) colors[i]= 1;
                  if (SMIBuffer[i]<SignalBuffer[i+1]) colors[i]=-1;
               }
               if (SMIBuffer[i]<levelOs) trend[i] = 1;
               if (SMIBuffer[i]>levelOb) trend[i] =-1;
               if (colors[i] == -1) PlotPoint(i,SMIBufferDa,SMIBufferDb,SMIBuffer);
      }
      
      //
      //
      //
      //
      //
      
      if (alertsOn)
      {
        if (alertsOnCurrent)
             int whichBar = 0;
        else     whichBar = 1; 
        if (trend[whichBar] != trend[whichBar+1])
        if (trend[whichBar] == 1)
              doAlert("crossed oversold");
        else  doAlert("crossed overbought");       
   }       
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
      {
         if (first[i+2] == EMPTY_VALUE) {
                first[i]   = from[i];
                first[i+1] = from[i+1];
                second[i]  = EMPTY_VALUE;
            }
         else {
                second[i]   =  from[i];
                second[i+1] =  from[i+1];
                first[i]    = EMPTY_VALUE;
            }
      }
   else
      {
         first[i]   = from[i];
         second[i]  = EMPTY_VALUE;
      }
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Blau - smi ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Blau - smi "),message);
             if (alertsSound)   PlaySound(soundFile);
      }
}